package cn.com.qmth.examcloud.core.oe.task.service.pipeline;

import cn.com.qmth.examcloud.commons.helpers.KeyValuePair;
import cn.com.qmth.examcloud.commons.helpers.pipeline.NodeExecuter;
import cn.com.qmth.examcloud.commons.helpers.pipeline.Storer;
import cn.com.qmth.examcloud.commons.helpers.pipeline.TaskContext;
import cn.com.qmth.examcloud.core.oe.task.service.ExamRecordDataService;
import cn.com.qmth.examcloud.core.oe.task.service.ExamingSessionService;
import cn.com.qmth.examcloud.examwork.api.ExamStudentCloudService;
import cn.com.qmth.examcloud.support.enums.ExamRecordStatus;
import cn.com.qmth.examcloud.support.examing.ExamRecordData;
import cn.com.qmth.examcloud.support.examing.ExamingSession;
import com.google.common.collect.Lists;
import com.sun.org.apache.xpath.internal.operations.Bool;
import org.apache.commons.io.FileUtils;
import org.apache.commons.lang3.RandomUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.io.File;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.Objects;

/**
 * @Description 自动交卷执行器
 * @Author lideyin
 * @Date 2019/12/17 16:39
 * @Version 1.0
 */
@Component
public class HandInExamExecutor implements NodeExecuter<Long, ExamRecordData, Long, ExamRecordData> {

    @Autowired
    private ExamingSessionService examingSessionService;
    @Autowired
    private ExamRecordDataService examRecordDataService;

    @Override
    public List<KeyValuePair<Long, ExamRecordData>> execute(Long key, ExamRecordData examRecordData, TaskContext context) throws Exception {

        List<KeyValuePair<Long, ExamRecordData>> resultList = new ArrayList<>();
        KeyValuePair<Long, ExamRecordData> keyValuePair = new KeyValuePair<>(key, examRecordData);

        //处理正在进行中的考试
        if (examRecordData.getExamRecordStatus() == ExamRecordStatus.EXAM_ING) {
            ExamingSession examingSession = examingSessionService.getExamingSession(examRecordData.getStudentId());

            // 如果考试会话不存在/超过考试时间/超过断点续考时间，自动交卷
            if (null == examingSession || isOverExamTime(examingSession) || isOverBreakpointTime(examingSession)) {
                Date now = new Date();

                //更改内存中的交卷状态
                examRecordData.setExamRecordStatus(ExamRecordStatus.EXAM_AUTO_HAND_IN);
                examRecordData.setCleanTime(now);

                examRecordDataService.saveExamRecordDataCache(examRecordData.getId(), examRecordData);

                keyValuePair.setValue(examRecordData);
                resultList.add(keyValuePair);

                // 删除redis会话
                examingSessionService.deleteExamingSession(examRecordData.getStudentId());
                return resultList;
            }
        }

        resultList.add(keyValuePair);
        return resultList;
    }

    /**
     * 是否超过考试时长
     *
     * @param examingSession
     * @return
     */
    private boolean isOverExamTime(ExamingSession examingSession) {
        return examingSession.getExamDuration() <= examingSession.getCost() * 1000;
    }

    /**
     * 是否超过断点续考时间
     *
     * @param examingSession
     * @return
     */
    private boolean isOverBreakpointTime(ExamingSession examingSession) {
        long now = System.currentTimeMillis();
        return now - examingSession.getActiveTime() >= examingSession.getExamReconnectTime().intValue() * 60 * 1000;
    }
}
